-- Volume scaling maximum for landing thud sound
-- Full landing thud volume is reached when you hit the ground at a speed of THUDMAX * FRACUNIT
-- Typical landing speed from a standing jump is 5 * FRACUNIT
local THUDMAX = 10
-- Minimum volume level for footsteps when walking
local MINSTEPVOLUME = 102

-- Footstep effect function
local function footSteps(player)
	-- Only play a sound at the start of an animation frame
	if player.mo.frame == player.prevframe
	-- Exception designed for Metal Sonic cause he only has one frame for most of the relevant animations
	and not (player.feet and player.feet["stiff"])
		return
	end
	
	local stepsound = nil
	local stepvolume = 255
	
	if player.mo.state == S_PLAY_WALK
	and (P_IsObjectOnGround(player.mo) or player.feet["airwalk"])
		-- Check if this is an appropriate frame to play footstep sounds on
		if (player.feet["walktiming"](player.mo.frame))
			-- Randomly pick one of available walking footstep sounds
			stepsound = player.feet["walksfx"][P_RandomRange(1, table.getn(player.feet["walksfx"]))]
		else
			return
		end
		-- Scale footstep volume to how close player is to running animation
		stepvolume = 255 * player.speed / player.runspeed
	elseif (player.mo.state == S_PLAY_RUN
	and (P_IsObjectOnGround(player.mo) or player.feet["airrun"]))
	-- Metal Sonic exception
	or player.mo.state == S_PLAY_FLOAT or player.mo.state == S_PLAY_FLOAT_RUN
		-- Check if this is an appropriate frame to play footstep sounds on
		if (player.feet["runtiming"](player.mo.frame))
			stepsound = player.feet["runsfx"][P_RandomRange(1, table.getn(player.feet["runsfx"]))]
		else
			return
		end
	-- Metal Sonic's so bloody special
	elseif player.mo.state == S_PLAY_DASH
		if (player.feet["dashtiming"](player.mo.frame))
			stepsound = player.feet["dashsfx"][P_RandomRange(1, table.getn(player.feet["dashsfx"]))]
		else
			return
		end
	-- Not walking or running, no footsteps
	else
		return
	end
	
	-- If anything went wrong, default to default footstep at 255
	S_StartSoundAtVolume(player.mo, stepsound or sfx_fstep0, max(stepvolume, MINSTEPVOLUME))
end

-- Landing thud effect function
local function landingThud(player)
	local thudsound = nil
	-- Scale landing thud sound to how fast they hit the ground
	local thudvolume = 255 * min(abs(player.prevmomz or 0), THUDMAX*FRACUNIT) / THUDMAX / FRACUNIT
	
	-- Player is moving on foot slowly or not at all
	if player.mo.state == S_PLAY_STND or player.mo.state == S_PLAY_WALK
		-- Play the slow landing thud
		thudsound = player.feet["landingsfx"]["slow"]
	-- Player is, or was, moving on foot quickly
	elseif player.mo.state == S_PLAY_RUN or player.mo.state == S_PLAY_SKID or player.mo.state == S_PLAY_DASH
		-- Default to slow landing thud if there isn't a fast thud
		thudsound = player.feet["landingsfx"]["fast"] or player.feet["landingsfx"]["slow"]
	-- Player is doing something else so their feet shouldn't be hitting the ground
	else
		return
	end
	
	S_StartSoundAtVolume(player.mo, thudsound or sfx_kc50, thudvolume or 255)
end

addHook("ThinkFrame", function()
	for player in players.iterate()
		if not (player.mo and player.mo.valid)
			continue
		end
		
		-- If you don't have any feet settings and default is silent,
		-- or if you have your own footstep code, skip mine
		if player.feet == nil
		or player.feet["handlesself"] == true
			return
		end
		
		-- Landing with a thud
		if P_IsObjectOnGround(player.mo) and player.prevground == false
			-- Landing thud effects
			landingThud(player)
		else
			-- Footstep effects - possibly including mid-air 'footsteps'
			footSteps(player)
		end
	end
end)

addHook("PostThinkFrame", function()
	for player in players.iterate()
		if not (player.mo and player.mo.valid)
			continue
		end
		
		--Set up vars for next frame's checks
		player.prevframe = player.mo.frame
		player.prevground = P_IsObjectOnGround(player.mo)
		player.prevmomz = player.mo.momz
		player.feet = nil
	end
end)